"""Version Information for truthound-airflow.

This module provides version information for the package.
The version follows Semantic Versioning (SemVer).
"""

__version__ = "0.1.0"
__version_tuple__ = (0, 1, 0)
